# urls.py

from django.urls import path
from django.views.decorators.csrf import csrf_exempt

from .apis import get_query_result


urlpatterns = [
    path('api/query_search_input/', csrf_exempt(get_query_result), name='get_search_input_query_result'),
]




from .query_function_register import API_REGISTER
from django.http import JsonResponse


def get_query_result(request):
    query_name = request.GET.get('query_name')
    query = request.GET.get('query')
    # get the query function
    query_function = API_REGISTER().get(query_name)
    
    # get the result
    try:
        options = query_function(query)
        result = []
        for option in options:
            result.append({'id': option.id, 'text': option.string})
        
    except Exception as e:
        result = {"error": str(e)}
        
    # return the result
    return JsonResponse(result, safe=False)



# fields.py
from typing import Any
from django import forms
from django.forms.widgets import Widget
from .widgets import SearchTextInput

class SearchCharField(forms.CharField):
    widget = SearchTextInput

    def __init__(self, query_function_name='', *args, **kwargs):
        assert query_function_name is not None, f"The query function name cannot be None."
        self.query_function_name = query_function_name
        super().__init__(*args, **kwargs)

    def widget_attrs(self, widget: Widget) -> Any:
        attrs = super().widget_attrs(widget)
        attrs.update({'query_function_name': self.query_function_name})
        return attrs
    
    
class InoutOptions():
    def __init__(self, id, string):
        self.id = id
        self.string = string


import inspect


class API_REGISTER():
    api_functions = {}
    
    
    def __new__(cls):
        if not hasattr(cls, 'instance'):
            cls.instance = super(API_REGISTER, cls).__new__(cls)
        return cls.instance
    
    
    # each function should only take one parameter, which is the query
    def register(self, function, name):
        assert callable(function), f"The function '{name}' is not callable."
        assert name not in self.api_functions, f"The function '{name}' is already registered."
        assert name is not None, f"The function name cannot be None."
        
        num_params = len(inspect.signature(function).parameters)
        assert num_params == 1, f"The function '{name}' should take only one parameter."
        
        self.api_functions[name] = function
    
    def get(self, name):
        return self.api_functions[name]
        
    
    
def register_api_function(name):
    def decorator(function):
        API_REGISTER().register(function, name)
        return function
    return decorator



from setuptools import setup, find_packages
# Remove the unused import statement for glob
# Add the templates folder to the build configuration
from setuptools import setup, find_packages


setup(
    name='django-search-input-field',
    version='0.1.0',
    license='MIT',
    author="Treyd Services AB",
    author_email='support@treyd.io',
    packages=find_packages(include=['django_search_input_field', 'django_search_input_field.*',
                                    'django_search_input_field.templates', 'django_search_input_field.templates.*']),
    include_package_data=True,
    package_data={'django_search_input_field': ['django_search_input_field/templates/*']},
    url='https://github.com/minaaaatef/django-search-input-form',
    install_requires=[

    ],
)
